package net.knarcraft.blacksmithvisuals.container;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.FileConfiguration;
import org.jetbrains.annotations.NotNull;

/**
 * An animation data container
 *
 * @param animateOffHand  <p>Whether to animate the off-hand at all</p>
 * @param animationDelay  <p>The delay between each time the animation should be attempted (max swing rate)</p>
 * @param animationChance <p>The probability of the animation triggering when the animation delay has passed</p>
 */
public record AnimationData(boolean animateOffHand, int animationDelay, int animationChance) {

    /**
     * Instantiates a new animation data object with sanity checks
     *
     * @param animateOffHand  <p>Whether to animate the off-hand at all</p>
     * @param animationDelay  <p>The delay between each time the animation should be attempted (max swing rate)</p>
     * @param animationChance <p>The probability of the animation triggering when the animation delay has passed</p>
     */
    public AnimationData {
        if (animationDelay < 1) {
            animationDelay = 1;
        }
        if (animationChance < 1) {
            animationChance = 1;
        }
        if (animationChance > 100) {
            animationChance = 100;
        }
    }

    /**
     * Loads sound data from the given configuration key
     *
     * @param configuration <p>The configuration to load values from</p>
     * @param rootKey       <p>The key to load the data from</p>
     * @return <p>The loaded sound data</p>
     */
    @NotNull
    public static AnimationData load(@NotNull FileConfiguration configuration,
                                     @NotNull String rootKey) throws InvalidConfigurationException {
        ConfigurationSection section = configuration.getConfigurationSection(rootKey);
        if (section == null) {
            throw new InvalidConfigurationException("Could not find the configuration section " + rootKey);
        }
        boolean animateOffHand = section.getBoolean("animateOffhand", true);
        int animationDelay = section.getInt("animationDelay", 5);
        int animationChance = section.getInt("animationChance", 20);

        return new AnimationData(animateOffHand, animationDelay, animationChance);
    }

}
