package net.knarcraft.blacksmithvisuals.container;

import net.knarcraft.blacksmithvisuals.BlacksmithVisuals;
import org.bukkit.Sound;
import org.bukkit.SoundCategory;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.FileConfiguration;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.logging.Level;

/**
 * A sound data container
 *
 * @param soundCategory <p>The category the sound should be played in</p>
 * @param sound         <p>The sound to be played</p>
 * @param volume        <p>The volume to play the sound at (> 0)</p>
 * @param pitch         <p>The pitch to play the sound at</p>
 * @param offsetTicks   <p>The amount of ticks to offset the sound by</p>
 * @param enabled       <p>Whether this sound is enabled</p>
 */
public record SoundData(@NotNull SoundCategory soundCategory, @NotNull Sound sound, float volume, float pitch,
                        int offsetTicks, boolean enabled) {

    /**
     * Instantiates a new sound data object with sanity checks
     *
     * @param soundCategory <p>The category the sound should be played in</p>
     * @param sound         <p>The sound to be played</p>
     * @param volume        <p>The volume to play the sound at (> 0)</p>
     * @param pitch         <p>The pitch to play the sound at</p>
     * @param offsetTicks   <p>The amount of ticks to offset the sound by</p>
     * @param enabled       <p>Whether this sound is enabled</p>
     */
    public SoundData {
        if (volume < 0) {
            volume = 1;
        }
        if (pitch < 0) {
            pitch = 0;
        } else if (pitch > 1) {
            pitch = 1;
        }
        Objects.requireNonNull(soundCategory);
        Objects.requireNonNull(sound);
    }

    /**
     * Loads sound data from the given configuration key
     *
     * @param configuration <p>The configuration to load values from</p>
     * @param rootKey       <p>The key to load the data from</p>
     * @return <p>The loaded sound data</p>
     */
    @NotNull
    public static SoundData load(@NotNull FileConfiguration configuration,
                                 @NotNull String rootKey) throws InvalidConfigurationException {
        ConfigurationSection section = configuration.getConfigurationSection(rootKey);
        if (section == null) {
            throw new InvalidConfigurationException("Could not find the configuration section " + rootKey);
        }
        boolean enabled = section.getBoolean("enabled", true);
        SoundCategory soundCategory = parseCategory(section.getString("soundCategory", "AMBIENT"));
        Sound sound = parseSound(section.getString("sound", "ENTITY_PIG_DEATH"));
        float pitch = (float) section.getDouble("pitch", 1);
        float volume = (float) section.getDouble("volume", 1);
        int offsetTicks = section.getInt("offset", 0);

        return new SoundData(soundCategory, sound, volume, pitch, offsetTicks, enabled);
    }

    /**
     * Safely parses a sound
     *
     * @param soundName <p>The name of the sound to parse</p>
     * @return <p>The parsed sound</p>
     */
    @NotNull
    private static Sound parseSound(@NotNull String soundName) {
        try {
            return Sound.valueOf(soundName);
        } catch (IllegalArgumentException exception) {
            BlacksmithVisuals.getInstance().getLogger().log(Level.WARNING, "Invalid sound in configuration: " +
                    soundName);
            return Sound.ENTITY_PIG_DEATH;
        }
    }

    /**
     * Safely parses a sound category
     *
     * @param categoryName <p>The name of the category to parse</p>
     * @return <p>The parsed category</p>
     */
    @NotNull
    private static SoundCategory parseCategory(@NotNull String categoryName) {
        try {
            return SoundCategory.valueOf(categoryName);
        } catch (IllegalArgumentException exception) {
            BlacksmithVisuals.getInstance().getLogger().log(Level.WARNING, "Invalid sound category in " +
                    "configuration: " + categoryName);
            return SoundCategory.AMBIENT;
        }
    }

}
