package net.knarcraft.blacksmithvisuals.listener;

import com.comphenix.protocol.PacketType;
import com.comphenix.protocol.ProtocolLibrary;
import com.comphenix.protocol.ProtocolManager;
import com.comphenix.protocol.events.PacketContainer;
import net.citizensnpcs.api.npc.NPC;
import net.citizensnpcs.trait.LookClose;
import net.knarcraft.blacksmith.event.ActionStartEvent;
import net.knarcraft.blacksmith.event.BlacksmithReforgeFailEvent;
import net.knarcraft.blacksmith.event.BlacksmithReforgeStartEvent;
import net.knarcraft.blacksmith.event.BlacksmithReforgeSucceedEvent;
import net.knarcraft.blacksmith.event.NPCSoundEvent;
import net.knarcraft.blacksmith.event.ScrapperSalvageFailEvent;
import net.knarcraft.blacksmith.event.ScrapperSalvageStartEvent;
import net.knarcraft.blacksmith.event.ScrapperSalvageSucceedEvent;
import net.knarcraft.blacksmithvisuals.BlacksmithVisuals;
import net.knarcraft.blacksmithvisuals.container.AnimationData;
import net.knarcraft.blacksmithvisuals.container.NPCData;
import net.knarcraft.blacksmithvisuals.container.SoundData;
import net.knarcraft.blacksmithvisuals.manager.ConfigurationManager;
import net.knarcraft.blacksmithvisuals.property.NPCPosition;
import net.knarcraft.blacksmithvisuals.property.SoundIdentifier;
import net.knarcraft.blacksmithvisuals.util.SoundHelper;
import net.knarcraft.knarlib.particle.ParticleConfig;
import net.knarcraft.knarlib.particle.ParticleMode;
import net.knarcraft.knarlib.util.ParticleHelper;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.scheduler.BukkitScheduler;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.UUID;
import java.util.logging.Level;

/**
 * A listener for blacksmith-related events
 */
public class BlacksmithListener implements Listener {

    private final Random random;
    private final ConfigurationManager configurationManager;
    private final Map<UUID, Double> oldLookRanges;

    /**
     * Instantiates a new blacksmith listener
     *
     * @param configurationManager <p>The configuration manager to get configuration values from</p>
     */
    public BlacksmithListener(@NotNull ConfigurationManager configurationManager) {
        this.random = new Random();
        this.configurationManager = configurationManager;
        this.oldLookRanges = new HashMap<>();
    }

    @EventHandler
    public void onDefaultSound(@NotNull NPCSoundEvent event) {
        event.setCancelled(true);
    }

    @EventHandler
    public void onReforgeStart(@NotNull BlacksmithReforgeStartEvent event) {
        runWorkingAnimation(event, SoundIdentifier.REFORGING_WORKING, this.configurationManager.getBlacksmithAnimationData());
    }

    @EventHandler
    public void onSalvageStart(@NotNull ScrapperSalvageStartEvent event) {
        runWorkingAnimation(event, SoundIdentifier.SALVAGING_WORKING, this.configurationManager.getScrapperAnimationData());
    }

    @EventHandler
    public void onReforgeSuccess(@NotNull BlacksmithReforgeSucceedEvent event) {
        SoundHelper.playSound(event.getNpc().getEntity(), this.configurationManager.getSoundData(SoundIdentifier.REFORGING_SUCCESS));
    }

    @EventHandler
    public void onSalvageSuccess(@NotNull ScrapperSalvageSucceedEvent event) {
        SoundHelper.playSound(event.getNpc().getEntity(), this.configurationManager.getSoundData(SoundIdentifier.SALVAGING_SUCCESS));
    }

    @EventHandler
    public void onReforgeFail(@NotNull BlacksmithReforgeFailEvent event) {
        SoundHelper.playSound(event.getNpc().getEntity(), this.configurationManager.getSoundData(SoundIdentifier.REFORGING_FAILURE));
    }

    @EventHandler
    public void onSalvageFail(@NotNull ScrapperSalvageFailEvent event) {
        SoundHelper.playSound(event.getNpc().getEntity(), this.configurationManager.getSoundData(SoundIdentifier.SALVAGING_FAILURE));
    }

    /**
     * Runs the working animation for an NPC
     *
     * @param event           <p>The action that started</p>
     * @param soundIdentifier <p>The identifier for the sound to play</p>
     * @param animationData   <p>The animation data for the animation to play</p>
     */
    private void runWorkingAnimation(@NotNull ActionStartEvent event, @NotNull SoundIdentifier soundIdentifier,
                                     @NotNull AnimationData animationData) {
        BlacksmithVisuals instance = BlacksmithVisuals.getInstance();
        BukkitScheduler scheduler = Bukkit.getScheduler();
        NPC npc = event.getNpc();

        NPCPosition npcPosition = NPCPosition.getFromMaterial(event.getCraftingStation());
        long delay = moveToWorkingPosition(npc, npcPosition);
        if (npc.hasTrait(LookClose.class)) {
            LookClose trait = npc.getTraitNullable(LookClose.class);
            this.oldLookRanges.put(npc.getUniqueId(), trait.getRange());
            trait.setRange(0);
        }
        long finishTime = event.getActionDurationTicks() - (2 * delay);

        scheduler.scheduleSyncDelayedTask(instance, () -> startWorkAnimation(npc.getEntity(),
                this.configurationManager.getSoundData(soundIdentifier), animationData, finishTime - 20, npcPosition), delay);
        scheduler.scheduleSyncDelayedTask(instance, () -> moveBack(event.getNpc()), finishTime);
    }

    /**
     * Moves an NPC back to its idle position
     *
     * @param npc <p>The NPC to move</p>
     */
    private void moveBack(@NotNull NPC npc) {
        if (!npc.isSpawned()) {
            return;
        }

        NPCData npcData = BlacksmithVisuals.getInstance().getNpcDataManager().getData(npc.getUniqueId());
        if (npcData == null) {
            return;
        }

        Location targetLocation = npcData.positions().get(NPCPosition.IDLE);
        if (!npc.getNavigator().canNavigateTo(targetLocation)) {
            BlacksmithVisuals.getInstance().getLogger().log(Level.WARNING, "Idle position for NPC " +
                    npc.getName() + " is unreachable!");
            return;
        }

        if (npc.hasTrait(LookClose.class)) {
            LookClose trait = npc.getTraitNullable(LookClose.class);
            Double oldRange = oldLookRanges.remove(npc.getUniqueId());
            if (oldRange != null) {
                trait.setRange(oldRange);
            }
        }

        Bukkit.getScheduler().scheduleSyncDelayedTask(BlacksmithVisuals.getInstance(), () ->
                npc.getEntity().teleport(targetLocation), getWalkTime(npc.getEntity().getLocation(), targetLocation));
        npc.getNavigator().setTarget(targetLocation);
    }

    /**
     * Moves a npc to its working position
     *
     * @param npc         <p>The NPC to move</p>
     * @param npcPosition <p>The npc position to move to</p>
     * @return <p>The time the move will take</p>
     */
    private long moveToWorkingPosition(@NotNull NPC npc, @Nullable NPCPosition npcPosition) {
        if (!npc.isSpawned() || npcPosition == null) {
            return 0;
        }

        NPCData npcData = BlacksmithVisuals.getInstance().getNpcDataManager().getData(npc.getUniqueId());
        if (npcData == null) {
            return 0;
        }

        Location targetLocation = npcData.positions().get(npcPosition);
        if (targetLocation == null && npcPosition != NPCPosition.WORKING_REPAIRABLE) {
            targetLocation = npcData.positions().get(NPCPosition.WORKING_REPAIRABLE);
        }
        if (targetLocation == null) {
            return 0;
        }

        if (!npc.getNavigator().canNavigateTo(targetLocation)) {
            BlacksmithVisuals.getInstance().getLogger().log(Level.WARNING, "Working position for NPC " +
                    npc.getName() + " is unreachable!");
            return 0;
        }
        Location finalTargetLocation = targetLocation;

        // Move NPC using Citizens path-finding
        Location current = npc.getEntity().getLocation().clone();
        npc.teleport(current.setDirection(targetLocation.toVector().subtract(current.toVector())), PlayerTeleportEvent.TeleportCause.PLUGIN);

        Bukkit.getScheduler().scheduleSyncDelayedTask(BlacksmithVisuals.getInstance(), () -> npc.getNavigator().setTarget(finalTargetLocation), 2);

        // Teleport the NPC tp get it in the exact final location
        long walkTime = getWalkTime(npc.getEntity().getLocation(), targetLocation);
        Bukkit.getScheduler().scheduleSyncDelayedTask(BlacksmithVisuals.getInstance(), () ->
                npc.getEntity().teleport(finalTargetLocation), walkTime);
        return walkTime;
    }

    /**
     * Starts the working animation and sound
     *
     * @param entity        <p>The entity to play the sound at</p>
     * @param soundData     <p>The sound data for the sound to play</p>
     * @param animationData <p>The animation data for the animation to play</p>
     * @param durationTicks <p>The duration of the work animation</p>
     * @param npcPosition   <p>The npc position to animate</p>
     */
    private void startWorkAnimation(@NotNull Entity entity, @NotNull SoundData soundData,
                                    @NotNull AnimationData animationData, long durationTicks,
                                    @Nullable NPCPosition npcPosition) {
        BlacksmithVisuals instance = BlacksmithVisuals.getInstance();
        BukkitScheduler scheduler = Bukkit.getScheduler();

        int playWorkSound = scheduler.scheduleSyncRepeatingTask(instance,
                () -> animateNPC(entity, soundData, animationData, npcPosition), 20, animationData.animationDelay());
        scheduler.scheduleSyncDelayedTask(instance, () -> scheduler.cancelTask(playWorkSound), durationTicks);
    }

    /**
     * Gets the time it would take to go from one location to another in ticks
     *
     * @param startLocation  <p>The location to start from</p>
     * @param targetLocation <p>The target location</p>
     * @return <p>The time in ticks</p>
     */
    private long getWalkTime(@NotNull Location startLocation, @NotNull Location targetLocation) {
        double distance = startLocation.distance(targetLocation);
        double WALK_SPEED = 4.317;
        return Math.round((distance / WALK_SPEED) * 20);
    }

    /**
     * Animates a npc's hand, and plays a sound
     *
     * @param entity        <p>The entity to animate and play the sound at</p>
     * @param soundData     <p>The sound to be played</p>
     * @param animationData <p>The animation to be played</p>
     * @param npcPosition   <p>The npc position to animate</p>
     */
    private void animateNPC(@NotNull Entity entity, @NotNull SoundData soundData, @NotNull AnimationData animationData,
                            @Nullable NPCPosition npcPosition) {
        if (random.nextInt(100) >= animationData.animationChance()) {
            return;
        }

        SoundHelper.playSound(entity, soundData);

        // Don't play disabled animations
        if (!animationData.animateOffHand()) {
            return;
        }

        if (soundData.offsetTicks() < 0) {
            Bukkit.getScheduler().scheduleSyncDelayedTask(BlacksmithVisuals.getInstance(),
                    () -> animateOffhand(entity), -soundData.offsetTicks());
        } else {
            animateOffhand(entity);
        }

        if (npcPosition == null) {
            npcPosition = NPCPosition.WORKING_REPAIRABLE;
        }
        for (int i = 0; i < random.nextInt(5) + 1; i++) {
            spawnSparkParticle(entity, npcPosition);
        }
    }

    /**
     * Spawns a spark particle at the given NPC's work station
     *
     * @param entity      <p>The entity whose working station should produce a spark</p>
     * @param npcPosition <p>The npc potion to spawn a particle for</p>
     */
    private void spawnSparkParticle(@NotNull Entity entity, @NotNull NPCPosition npcPosition) {
        double randomX = this.random.nextDouble(-1, 1);
        double randomY = this.random.nextDouble(0.1, 1);
        double randomZ = this.random.nextDouble(-1, 1);
        Particle particle;
        if (npcPosition == NPCPosition.WORKING_CRAFTING) {
            particle = Particle.ELECTRIC_SPARK;
        } else {
            particle = Particle.FLAME;
        }
        Location spawnLocation = entity.getLocation().clone().getBlock().getRelative(entity.getFacing()).getLocation().add(0.5, 0, 0.5);
        ParticleConfig particleConfig = new ParticleConfig(ParticleMode.SINGLE, particle, 0, 0, 1.1, randomX, randomY, randomZ, 0.1);
        ParticleHelper.spawnParticle(entity.getWorld(), spawnLocation, particleConfig, 1);
    }

    /**
     * Animates an NPC's offhand
     *
     * @param entity <p>The entity to animate</p>
     */
    private void animateOffhand(@NotNull Entity entity) {
        ProtocolManager manager = ProtocolLibrary.getProtocolManager();
        PacketContainer packet = manager.createPacket(PacketType.Play.Server.ANIMATION);
        packet.getIntegers().write(0, entity.getEntityId());
        packet.getIntegers().write(1, 3);

        for (Player player : Bukkit.getOnlinePlayers()) {
            if (player.getWorld().equals(entity.getWorld())) {
                manager.sendServerPacket(player, packet);
            }
        }
    }


}
