package net.knarcraft.blacksmithvisuals.manager;

import net.knarcraft.blacksmithvisuals.BlacksmithVisuals;
import net.knarcraft.blacksmithvisuals.container.NPCData;
import net.knarcraft.blacksmithvisuals.property.NPCPosition;
import org.bukkit.Location;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.logging.Level;

/**
 * A manager for NPC data
 */
public class NPCDataManager {

    private final Map<UUID, NPCData> npcDataMap;
    private static final File configurationFile = new File(BlacksmithVisuals.getInstance().getDataFolder(), "data.yml");

    /**
     * Instantiates a new NPC data manager
     *
     * @param npcDataMap <p>The map containing existing NPC data</p>
     */
    public NPCDataManager(@NotNull Map<UUID, NPCData> npcDataMap) {
        this.npcDataMap = npcDataMap;
    }

    /**
     * Gets the NPC data for the NPC with the given id
     *
     * @param npcId <p>The id of the NPC to get data for</p>
     * @return <p>The NPC data, or null if not set</p>
     */
    @Nullable
    public NPCData getData(@NotNull UUID npcId) {
        return this.npcDataMap.get(npcId);
    }

    /**
     * Adds new NPC data to an NPC
     *
     * @param npcId   <p>The id of the NPC to add data to</p>
     * @param npcData <p>The data to set</p>
     */
    public void putData(@NotNull UUID npcId, @NotNull NPCData npcData) {
        this.npcDataMap.put(npcId, npcData);
        try {
            this.save();
        } catch (IOException exception) {
            BlacksmithVisuals.getInstance().getLogger().log(Level.SEVERE, "Unable to save NPC data. Error was: " +
                    exception);
        }
    }

    /**
     * Saves this data manager's data to disk
     *
     * @throws IOException <p>If unable to write the configuration file</p>
     */
    public void save() throws IOException {
        if (!configurationFile.exists() && !configurationFile.createNewFile()) {
            throw new FileNotFoundException("data.yml could not be found or created. Please create it manually.");
        }
        FileConfiguration configuration = new YamlConfiguration();
        for (Map.Entry<UUID, NPCData> entry : npcDataMap.entrySet()) {
            ConfigurationSection npcSection = configuration.createSection(entry.getKey().toString());
            npcSection.set("workingPositionNetherite", entry.getValue().positions().get(NPCPosition.WORKING_NETHERITE));
            npcSection.set("workingPositionCrafting", entry.getValue().positions().get(NPCPosition.WORKING_CRAFTING));
            npcSection.set("workingPositionRepairable", entry.getValue().positions().get(NPCPosition.WORKING_REPAIRABLE));
            npcSection.set("idlePosition", entry.getValue().positions().get(NPCPosition.IDLE));
        }
        configuration.save(configurationFile);
    }

    /**
     * Loads a data manager from disk
     *
     * @return <p>The loaded data manager</p>
     * @throws IOException <p>If unable to load the configuration file</p>
     */
    @NotNull
    public static NPCDataManager load() throws IOException {
        if (!configurationFile.exists() && !configurationFile.createNewFile()) {
            throw new FileNotFoundException("data.yml could not be found or created. Please create it manually.");
        }
        Map<UUID, NPCData> npcDataMap = new HashMap<>();

        FileConfiguration configuration = YamlConfiguration.loadConfiguration(configurationFile);
        for (String configurationSectionKey : configuration.getKeys(false)) {
            ConfigurationSection section = configuration.getConfigurationSection(configurationSectionKey);
            if (section == null) {
                continue;
            }
            Map<NPCPosition, Location> locationMap = new HashMap<>();

            locationMap.put(NPCPosition.WORKING_NETHERITE, section.getLocation("workingPositionNetherite"));
            locationMap.put(NPCPosition.WORKING_CRAFTING, section.getLocation("workingPositionCrafting"));
            locationMap.put(NPCPosition.WORKING_REPAIRABLE, section.getLocation("workingPositionRepairable"));
            locationMap.put(NPCPosition.IDLE, section.getLocation("idlePosition"));
            npcDataMap.put(UUID.fromString(configurationSectionKey), new NPCData(locationMap));
        }
        return new NPCDataManager(npcDataMap);
    }

}
