package net.knarcraft.blacksmithvisuals.property;

import net.citizensnpcs.api.trait.Trait;
import net.knarcraft.blacksmith.trait.BlacksmithTrait;
import net.knarcraft.blacksmith.trait.ScrapperTrait;
import org.bukkit.Material;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * A definable NPC position
 */
public enum NPCPosition {

    /**
     * The position the NPC should be in while idle
     */
    IDLE("idle", List.of(BlacksmithTrait.class, ScrapperTrait.class)),

    /**
     * The position the NPC should be in while undoing netherite or armor trims
     */
    WORKING_NETHERITE("netherite-workstation", List.of(ScrapperTrait.class)),

    /**
     * The position the NPC should be in while reforging armor, weapons or tools
     */
    WORKING_REPAIRABLE("repairable-workstation", List.of(BlacksmithTrait.class, ScrapperTrait.class)),

    /**
     * The position the NPC should be in while un-crafting items
     */
    WORKING_CRAFTING("crafting-workstation", List.of(ScrapperTrait.class)),
    ;

    private final String positionName;
    private final List<Class<? extends Trait>> applicableTraits;

    /**
     * Instantiates a new NPC position
     *
     * @param positionName     <p>The user-facing name of the position</p>
     * @param applicableTraits <p>The traits the position can be applied to</p>
     */
    NPCPosition(@NotNull String positionName, @NotNull List<Class<? extends Trait>> applicableTraits) {
        this.positionName = positionName;
        this.applicableTraits = applicableTraits;
    }

    /**
     * Gets the user-friendly name for this position
     *
     * @return <p>The position name</p>
     */
    @NotNull
    public String getPositionName() {
        return this.positionName;
    }

    /**
     * Gets the NPC position the work station's material
     *
     * @param material <p>The material of the workstation</p>
     * @return <p>The corresponding NPC position</p>
     */
    @Nullable
    public static NPCPosition getFromMaterial(@NotNull Material material) {
        return switch (material) {
            case CRAFTING_TABLE -> NPCPosition.WORKING_CRAFTING;
            case ANVIL -> NPCPosition.WORKING_REPAIRABLE;
            case SMITHING_TABLE -> NPCPosition.WORKING_NETHERITE;
            default -> null;
        };
    }

    /**
     * Gets an NPC position from the given user input
     *
     * @param input <p>The input to get an NPC position from</p>
     * @return <p>The NPC position, or null if not recognized</p>
     */
    @Nullable
    public static NPCPosition fromInput(@NotNull String input) {
        String cleaned = input.toLowerCase().replace("_", "-");
        for (NPCPosition position : NPCPosition.values()) {
            if (position.getPositionName().equalsIgnoreCase(cleaned)) {
                return position;
            }
        }
        return null;
    }

    /**
     * Gets whether this position type is applicable to the given trait
     *
     * @param trait <p>The trait to check</p>
     * @return <p>True if this position is applicable</p>
     */
    public boolean isApplicable(@NotNull Class<? extends Trait> trait) {
        return this.applicableTraits.contains(trait);
    }

}
